@@ -8,7 +8,7 @@ use LWP::UserAgent;
 use JSON;
 use List::Util qw(first);
 
-our $VERSION = 0.4;
+our $VERSION = 0.5;
 
 sub new {
     my ($class, %params) = @_;
@@ -61,6 +61,64 @@ sub get_policy_id {
     return $policy->{id};
 }
 
+sub import_policy {
+    my ($self, %params) = @_;
+
+    croak "missing file name parameter" unless $params{id};
+
+    my $result = $self->_post("/policies/import", %params);
+    return $result;
+}
+
+sub get_policy_details {
+    my ($self, %params) = @_;
+
+    croak "missing id parameter" unless $params{id};
+
+	my $policy_id = delete $params{id};
+
+    my $result = $self->_get("/policies/$policy_id");
+    return $result;
+}
+
+sub delete_policy {
+    my ($self, %params) = @_;
+
+    croak "missing Policy id parameter" unless $params{id};
+
+	my $policy_id = delete $params{id};
+    my $result = $self->_delete("/policies/$policy_id");
+
+    return $result;
+}
+
+# Experimental
+sub configure_policy {
+    my ($self, %params) = @_;
+
+    croak "missing id parameter" unless $params{id};
+    croak "missing uuid parameter" unless $params{uuid};
+    croak "missing settings parameter" unless $params{settings};
+
+	my $policy_id = delete $params{id};
+    my $result = $self->_put("/policies/$policy_id", %params);
+
+    return $result;
+}
+
+# Experimental
+sub create_policy {
+    my ($self, %params) = @_;
+
+    croak "missing uuid parameter" unless $params{uuid};
+    croak "missing settings parameter" unless $params{settings};
+
+	my $uuid = delete $params{uuid};
+    my $result = $self->_post("/policies", %params);
+
+    return $result;
+}
+
 sub create_scan {
     my ($self, %params) = @_;
 
@@ -320,7 +378,7 @@ sub list_plugin_families {
     my ($self) = @_;
 
     my $result  = $self->_get("/plugins/families");
-    return $result ? @{$result} : ();
+    return $result->{families} ? @{$result->{families}} : ();
 }
 
 sub get_plugin_family_details {
@@ -354,6 +412,17 @@ sub get_scanner_id {
     return $scanner->{id};
 }
 
+sub file_upload {
+    my ($self, %params) = @_;
+
+    croak "missing file name" unless $params{file};
+
+	my $file = delete $params{file};
+
+	my $result = $self->_post_file('/file/upload', $file);
+	return $result;
+}
+
 sub _get {
     my ($self, $path, %params) = @_;
 
@@ -417,6 +486,30 @@ sub _post {
     }
 }
 
+sub _post_file {
+    my ($self, $path, $file) = @_;
+
+    my $response = $self->{agent}->post(
+        $self->{url} . $path,
+        'Content-Type' => 'multipart/form-data',
+        'Content'      => [
+			Filedata => [$file]
+		]
+    );
+
+    my $result = eval { from_json($response->content()) };
+
+    if ($response->is_success()) {
+        return $result;
+    } else {
+        if ($result) {
+            croak "server error: " . $result->{error};
+        } else {
+            croak "communication error: " . $response->message()
+        }
+    }
+}
+
 sub _put {
     my ($self, $path, %params) = @_;
 
@@ -534,6 +627,30 @@ See L<https://your.nessus.server:8834/nessus6-api.html#/resources/policies/list>
 
 Returns the identifier for the policy with given name.
 
+=head2 $nessus->get_policy_details(id => $policy_id)
+
+Returns a reference to a hash with all settings and parameters for a given scan policy.
+
+See L<https://your.nessus.server:8834/api#/resources/policies/details> for details.
+
+=head2 $nessus->import_policy(id => $file_id)
+
+Returns reference to hash with name and identifier of the policy imported.
+NB $file_id must be a valid identifier to a file uploaded to the Nessus server,
+e.g. with method file_upload()
+
+Example:
+$result = $nessus->import_policy(file => $fileuploaded);
+print "Policy imported: " . $result->{'name'} . "\n";
+
+See L<https://your.nessus.server:8834/api#/resources/file/upload> for details.
+
+=head2 $nessus->delete_policy(id => $policy_id)
+
+Deletes a given scan policy off the Nessus server
+
+See L<https://your.nessus.server:8834/api#/resources/policies/delete> for details.
+
 =head2 $nessus->list_scanners()
 
 Returns the scanner list.
@@ -651,7 +768,7 @@ See L<https://your.nessus.server:8834/nessus6-api.html#/resources/editor/list> f
 
 Returns the identifier for template with given name.
 
-=head2 $nessus->get_plugin_details( plugin_id => $plugin_id )
+=head2 $nessus->get_plugin_details( id => $plugin_id )
 
 returns the details of a plugin
 
@@ -663,7 +780,7 @@ returns a list of plugin families
 
 See L<https://your.nessus.server:8834/nessus6-api.html#/resources/plugins/families> for details.
 
-=head2 $nessus->get_plugin_family_details( )
+=head2 $nessus->get_plugin_family_details( id => $family_id )
 
 returns the details about a plugin family
 
@@ -673,6 +790,17 @@ See L<https://your.nessus.server:8834/nessus6-api.html#/resources/plugins/family
 
 returns the identifier for the scanner with given name.
 
+=head2 $nessus->file_upload(file => $file)
+
+Uploads a file to the Nessus server.
+Returns a reference to hash with identifier to the uploaded file.
+
+Example:
+my $result = $nessus->file_upload(file => $file);
+my $fileuploaded = $result->{'fileuploaded'};
+
+See L<https://your.nessus.server:8834/api#/resources/file/upload> for details.
+
 =head1 LICENSE
 
 This program is free software: you can redistribute it and/or modify
